# IPos project

This repository is a centralized version of architecture for IPos software (Indoor Positioning Software Framework for Intralogistics Applications).


## Architecture
![](PRG_talk-Layers.jpg)
    - [diagram](https://drive.google.com/file/d/12oIwl0zkeT2bsS8MUbbtSgFbbirG8Sz1/view?usp=sharing)


## 1. How to run? General 

1. Start [hivemq](http://www.hivemq.com/demos/websocket-client/). It's a client for an external test MQTT-broker.
    1. Establish a connection with the standard parameters (WS coonection).
    2. Add new subscriptions with `Add New Topic Subscription` button
        - `ipos52/user/position`: responses
        - `ipos52/sensor/measurement/position`: requests
2. Start Spring boot project from this repository
    - from console (`./gradlew bootRun`) or your favorite IDE
3. Return to a web-browser tab with an external MQTT client
    1. Send valid json representation of position in `ipos52/sensor/measurement/position` topic
        - Message example: `{"x": 3.0,"y": 2.0,"z": 3.0,"accuracy": 0.9}`
    2. If successful, the same message should come from `ipos52/user/position` topic.


## 2. 🚧 How to run? Demo

### Topics
- `ipos52/user/monitoring-request`
  - Monitoring requests from users
  - Example:
    - ```json
      {
        "frameId": "frame-1",
        "delta": 0.1,
        "updateFrequency": 0.1,
        "type": ["ROBOT"],
        "id": ["obj-123"],
        "fusionStrategy": "f-strategy",
        "exit_notification": false,
        "properties": ["prop-1"],
        "monitoringTaskId": "mon-id-1234",
        "requestorProtocol": "MQTT"
      }
      ```
    - should be valid for `IposMonitoringRequest` from `SimpleScene.proto`
- `ipos52/sensor/measurement/position`
- `ipos52/user/position`
- `local/user/monitoring-request`
- `local/user/position`


![](demo.v1.jpg)


### Extra
- Internal database: [localhost:8077/h2-console](http://localhost:8077/h2-console)
  - server 8077 port from `application.yml`
  - JDBC URL: see logs. Example: `jdbc:h2:mem:1a7bd898-e992-4a5c-9a3e-ed8d9eb2ab37`
  - [ ] TODO: fix database schema


### Notes
- MQTT
  - Ensure topic levels only use lowercase letters, numbers, and dashes. Customers should avoid camel casing and using hard to debug characters such as spaces [5].
  - Topic's names. From general to specific [5].

    
## Links

1. [Messaging with JMS](https://spring.io/guides/gs/messaging-jms/)
2. [Using WebSocket to build an interactive web application](https://spring.io/guides/gs/messaging-stomp-websocket/)
3. [Java Message Service API via Apache ActiveMQ Classic](https://spring.io/guides/gs/messaging-jms/)
4. [MQTT Topics & Best Practices](https://www.hivemq.com/blog/mqtt-essentials-part-5-mqtt-topics-best-practices/)
5. [MQTT Design Best Practices](https://docs.aws.amazon.com/whitepapers/latest/designing-mqtt-topics-aws-iot-core/mqtt-design-best-practices.html)