package ipos.project.SensorValueIntegration.api;

import com.google.protobuf.InvalidProtocolBufferException;
import ipos.models.SimpleScene.IposPosition;
import ipos.project.config.mqtt.Handler;
import ipos.project.config.mqtt.MqttListener;
import ipos.project.mapper.ProtoMapper;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.core.JmsTemplate;


// subscribe to the topic
@MqttListener("ipos52/sensor/measurement/position")
public class MqttPositionHandler implements Handler {

    private final Logger LOG = LoggerFactory.getLogger(getClass());
    JmsTemplate jmsTemplate;

    @Autowired
    public MqttPositionHandler(JmsTemplate jmsTemplate) {
        this.jmsTemplate = jmsTemplate;
    }

    // method that handle new message from the topic
    public void handle(MqttMessage message) {
        try {
            IposPosition pos = ProtoMapper.fromJson(message.toString(), IposPosition.class);
            assert pos != null;
            LOG.info("IposPosition received: \n" + pos);
            this.jmsTemplate.convertAndSend("local/user/position", pos); // submit position to the internal broker
        } catch (InvalidProtocolBufferException e) {
            LOG.error("Invalid mqtt message:", e);
        }
    }
}
