package ipos.project.UseCaseController;

import ipos.project.data.datamodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;
import ipos.models.SimpleScene.IposPosition;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Component
public class PositionMonitoring {
    private final Logger LOG = LoggerFactory.getLogger(getClass());

    public WorldModel wm;
    private final IPos_DatamodelFactory modelFactory = IPos_DatamodelFactory.eINSTANCE;

    public void initialize(){
        wm = modelFactory.createWorldModel();
        wm.getReferenceSystem().set(0, createReferenceSystem("ROOT", "ROOT", null));
        wm.getPois().set(0, createPOI(createPosition(createPoint3D(1.4f, 2.0f, 0f), 1f, wm.getReferenceSystem().get(0)), "UWBToken1", new HashMap<String,String>()));
        wm.getPois().set(1, createPOI(createPosition(createPoint3D(3.7f, -2.0f, 0f), 1f, wm.getReferenceSystem().get(0)), "UWBToken2", new HashMap<String,String>()));
        wm.getPois().set(2, createPOI(createPosition(createPoint3D(-2.4f, -0.5f, 0f), 1f, wm.getReferenceSystem().get(0)), "UWBToken3", new HashMap<String,String>()));
        wm.getZoneMap().set(0, modelFactory.createZoneMap());
        wm.getZoneMap().get(0).getZone().set(0, createZone("cobot1_warning_zone", "cobot1_warning_zone", createPlacing(createPosition(createPoint3D(1.1f, 0.9f, 0f), 1f, wm.getReferenceSystem().get(0)), createOrientation(0f, 0f, 0f, 0f)), createSpace(3.0f, 3.0f, 3.0f)));
        wm.getAgent().set(0, createAgent("Employee1", "HUMAN", null ));
        wm.getAgent().get(0).getLObject().set(0, createLobject("UWB_1", "UWB", wm.getAgent().get(0), null));
    }

    private POI createPOI(Position position, String description, Map<String, String> data){
        POI poi = modelFactory.createPOI();
        poi.setPosition(position);
        poi.setDescription(description);
        poi.setData(data);
        return poi;
    }

    private Agent createAgent(String id, String type, List<LocalizableObject> lobjects){
        Agent agent = modelFactory.createAgent();
        agent.setId(id);
        agent.setAgentType(type);
        agent.getLObject().addAll(lobjects);
        return agent;
    }

    private LocalizableObject createLobject(String id, String sensorType, Agent agent, Placing currentPlacing){
        LocalizableObject lobject = modelFactory.createLocalizableObject();
        lobject.setAgent(agent);
        lobject.setId(id);
        lobject.setSensorType(sensorType);
        lobject.setCurrentPlacing(currentPlacing);
        return lobject;
    }

    private Placing createPlacing(Position position, Orientation orientation){
        Placing placing = modelFactory.createPlacing();
        placing.setPosition(position);
        placing.setOrientation(orientation);
        return placing;
    }

    private Position createPosition(Point3D point3D, Float accuracy, ReferenceSystem refSystem){
        Position position = modelFactory.createPosition();
        position.setPoint(point3D);
        Gaussian accrcy = modelFactory.createGaussian();
        accrcy.setConfidenceInterval(accuracy);
        position.setAccuracy(accrcy);
        position.setReferenceSystem(refSystem);
        return position;
    }

    private Point3D createPoint3D(float x, float y, float z){
        Point3D point3D = modelFactory.createPoint3D();
        point3D.setX(x);
        point3D.setY(y);
        point3D.setZ(z);
        return point3D;
    }

    private ReferenceSystem createReferenceSystem(String name, String id, Placing origin){
        ReferenceSystem refSystem = modelFactory.createReferenceSystem();
        refSystem.setName(name);
        refSystem.setId(id);
        refSystem.setOrigin(origin);
        return refSystem;
    }

    private Zone createZone(String name, String id, Placing centrePoint, Space space){
        Zone zone = modelFactory.createZone();
        zone.setName(name);
        zone.setId(id);
        zone.setCentrePoint(centrePoint);
        zone.getSpace().add(space);
        return zone;
    }

    private Space createSpace(float x, float y, float z){
        Space space = modelFactory.createSpace();
        space.setX(x);
        space.setY(y);
        space.setZ(z);
        return space;
    }

    private Quaternion createOrientation(float x, float y, float z, float w){
        Quaternion quaternion = modelFactory.createQuaternion();
        quaternion.setX(x);
        quaternion.setY(y);
        quaternion.setZ(z);
        quaternion.setW(w);
        return quaternion;
    }

    @JmsListener(destination = "local/user/position", containerFactory = "jmsListenFactory")
    public void receiveMessage(IposPosition pos) {
        LOG.info("Internal: " + pos);
    }

    @JmsListener(destination = "local/user/monitoring-request", containerFactory = "jmsListenFactory")
    public void receiveMessage(MonitoringRequest monReq) {
        LOG.info("Internal: " + monReq);
        // SimpleSceneIntegration muss MQTT-topic gemäß monreq.MonitoringTaskId einrichten
    }

}
